#!/usr/bin/env python
"""
Simple example for rendering a graph with the Django web framework.
See
http://www.djangoproject.com/
and
http://www.djangobook.com/en/beta/chapter11/

"""
#    Copyright (C) 2007 by
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Manos Renieris, http://www.cs.brown.edu/~er/
#    Distributed with BSD license.
#    All rights reserved, see LICENSE for details.


__author__ = """Aric Hagberg (hagberg@lanl.gov)"""

try:
    from django.http import HttpResponse
except ImportError:  # this won't run without Django, print message
    print("Django not found.")


def pygraphviz_graph(request):
    import pygraphviz as P

    A = P.AGraph()  # init empty graph
    # set some default node attributes
    A.node_attr["style"] = "filled"
    A.node_attr["shape"] = "circle"
    # Add edges (and nodes)
    A.add_edge(1, 2)
    A.add_edge(2, 3)
    A.add_edge(1, 3)
    A.layout()  # layout with default (neato)
    png = A.draw(format="png")  # draw png
    return HttpResponse(png, mimetype="image/png")


if __name__ == "__main__":
    print(
        """This code works with the Django web framework
and should run as a django "view".
See djangoproject.com for info.

"""
    )



"""
Star
====

Create and draw a star with varying node properties.
"""
#    Copyright (C) 2006 by
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Manos Renieris, http://www.cs.brown.edu/~er/
#    Distributed with BSD license.
#    All rights reserved, see LICENSE for details.


__author__ = """Aric Hagberg (hagberg@lanl.gov)"""

from pygraphviz import *

A = AGraph()

# set some default node attributes
A.node_attr["style"] = "filled"
A.node_attr["shape"] = "circle"
A.node_attr["fixedsize"] = "true"
A.node_attr["fontcolor"] = "#FFFFFF"

# make a star in shades of red
for i in range(1, 16):
    A.add_edge(0, i)
    n = A.get_node(i)
    n.attr["fillcolor"] = "#%2x0000" % (i * 16)
    n.attr["height"] = "%s" % (i / 16.0 + 0.5)
    n.attr["width"] = "%s" % (i / 16.0 + 0.5)

print(A.string())  # print to screen
A.write("star.dot")  # write to simple.dot
A.draw("star.png", prog="circo")  # draw to png using circo layout



import pygraphviz as pgv

stringify = pgv.testing.stringify


def test_default_attributes():
    A = pgv.AGraph()
    A.graph_attr["label"] = "test"
    A.graph_attr["spam"] = "eggs"
    assert "label" in A.graph_attr
    assert A.graph_attr["label"] == "test"
    assert A.graph_attr.keys() == ["label", "spam"]
    graph_attrs = [("label", "test"), ("spam", "eggs")]
    assert sorted(list(A.graph_attr.iteritems())) == graph_attrs
    ans = """strict graph { graph [label=test, spam=eggs]; }"""
    assert stringify(A) == " ".join(ans.split())

    A.graph_attr["label"] = ""
    A.graph_attr["spam"] = ""
    ans = """strict graph { }"""
    assert stringify(A) == " ".join(ans.split())

    A.graph_attr["label"] = "test"
    del A.graph_attr["label"]
    ans = """strict graph { }"""
    assert stringify(A) == " ".join(ans.split())


def test_graph_defaults():
    A = pgv.AGraph(rankdir="LR", pack="true")
    ans = """strict graph { graph [pack=true, rankdir=LR]; }"""
    assert stringify(A) == " ".join(ans.split())


def test_node_defaults():
    A = pgv.AGraph()
    A.node_attr["label"] = "test"
    assert "label" in A.node_attr
    assert A.node_attr["label"] == "test"
    assert A.node_attr.keys() == ["label"]
    assert A.node_attr == {"label": "test"}
    assert list(A.node_attr.iteritems()) == [("label", "test")]
    ans = """strict graph { node [label=test]; }"""
    assert stringify(A) == " ".join(ans.split())

    A.node_attr["label"] = ""
    ans = """strict graph { }"""
    assert stringify(A) == " ".join(ans.split())

    A.node_attr["label"] = "test"
    del A.node_attr["label"]
    ans = """strict graph { }"""
    assert stringify(A) == " ".join(ans.split())

    A.graph_attr["fontname"] = "graph font"
    A.node_attr["fontname"] = "node font"
    A.edge_attr["fontname"] = "edge font"
    ans = """strict graph {
          graph [fontname="graph font"];
          node [fontname="node font"];
          edge [fontname="edge font"];
        }"""
    assert stringify(A) == " ".join(ans.split())


def test_edge_defaults():
    A = pgv.AGraph()
    A.edge_attr["label"] = "test"
    assert "label" in A.edge_attr
    assert A.edge_attr["label"] == "test"
    assert A.edge_attr.keys() == ["label"]
    assert A.edge_attr == {"label": "test"}
    assert list(A.edge_attr.iteritems()) == [("label", "test")]

    ans = """strict graph { edge [label=test]; } """
    assert stringify(A) == " ".join(ans.split())

    A.edge_attr["label"] = ""
    ans = """strict graph { }"""
    assert stringify(A) == " ".join(ans.split())

    A.edge_attr["label"] = "test"
    del A.edge_attr["label"]
    ans = """strict graph { }"""
    assert stringify(A) == " ".join(ans.split())



"""
Basic
=====

A simple example to create a graphviz dot file and draw a graph.
"""
#    Copyright (C) 2006 by
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Manos Renieris, http://www.cs.brown.edu/~er/
#    Distributed with BSD license.
#    All rights reserved, see LICENSE for details.


__author__ = """Aric Hagberg (hagberg@lanl.gov)"""

import pygraphviz as pgv

A = pgv.AGraph()

A.add_edge(1, 2)
A.add_edge(2, 3)
A.add_edge(1, 3)

print(A.string())  # print to screen
A.write("simple.dot")  # write to simple.dot

B = pgv.AGraph("simple.dot")  # create a new graph from file
B.layout()  # layout with default (neato)
B.draw("simple.png")  # draw png



"""
Attributes
----------

Example illustrating how to set node, edge, and graph attributes for
visualization.
"""

import pygraphviz as pgv

# strict (no parallel edges)
# digraph
# with attribute rankdir set to 'LR'
A = pgv.AGraph(directed=True, strict=True, rankdir="LR")
# add node 1 with color red
A.add_node(1, color="red")
A.add_node(5, color="blue")
# add some edges
A.add_edge(1, 2, color="green")
A.add_edge(2, 3)
A.add_edge(1, 3)
A.add_edge(3, 4)
A.add_edge(3, 5)
A.add_edge(3, 6)
A.add_edge(4, 6)
# adjust a graph parameter
A.graph_attr["epsilon"] = "0.001"
print(A.string())  # print dot file to standard output
A.layout("dot")  # layout with dot
A.draw("foo.png")  # write to file



def stringify(agraph):
    result = agraph.string().split()
    if '""' in result:
        result.remove('""')

    some = " ".join(result)
    while " ]" in some:
        some = some.replace(" ]", "]")
    if "\\N" in some:
        num_nattrs = len(agraph.node_attr)
        if num_nattrs == 1:
            some = some.replace('node [label="\\N"];', "")
        else:
            if "[label" in some:
                some = some.replace('[label="\\N", ', "[")
            else:
                some = some.replace(', label="\\N"', "")
    result = some.split()
    return " ".join(result)



"""
Subgraph
========

Specify a subgraph in pygraphviz.
"""

import pygraphviz as pgv

A = pgv.AGraph()
# add some edges
A.add_edge(1, 2)
A.add_edge(2, 3)
A.add_edge(1, 3)
A.add_edge(3, 4)
A.add_edge(3, 5)
A.add_edge(3, 6)
A.add_edge(4, 6)
# make a subgraph with rank='same'
B = A.add_subgraph([4, 5, 6], name="s1", rank="same")
B.graph_attr["rank"] = "same"
print(A.string())  # print dot file to standard output
A.draw("subgraph.png", prog="neato")
